package com.persagy.apm.diagnose.indicatorrecord.model;

import com.persagy.framework.ems.data.pojo.hbase.MonitorIndicatorRecord;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.diagnose.indicatorrecord.model.vo.*;
import com.persagy.apm.diagnose.indicatorrecord.model.dto.*;

import java.util.List;

/**
 * 监测指标历史记录(MonitorIndicatorRecord) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-10 00:13:27
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertMonitorIndicatorRecordTool {

    ConvertMonitorIndicatorRecordTool INSTANCE = Mappers.getMapper(ConvertMonitorIndicatorRecordTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param monitorIndicatorRecord do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-10 00:13:27
    */
    @Mappings({ @Mapping(source = "dataTime", target = "date") ,@Mapping(source = "dataValue", target = "value")})
    MonitorIndicatorRecordItemVO convert2ResponseItemDTO(MonitorIndicatorRecord monitorIndicatorRecord);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param monitorIndicatorRecord do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-10 00:13:27
    */
    @Mappings({ @Mapping(source = "dataTime", target = "date") ,@Mapping(source = "dataValue", target = "value")})
    MonitorIndicatorRecordListItemVO convert2ResponseListItemDTO(MonitorIndicatorRecord monitorIndicatorRecord);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param monitorIndicatorRecordList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-10 00:13:27
    */
    List<MonitorIndicatorRecordListItemVO> convert2List(List<MonitorIndicatorRecord> monitorIndicatorRecordList);
    
    /**
    * addDTO转换为do
    *
    * @param addMonitorIndicatorRecordDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-10 00:13:27
    */
    MonitorIndicatorRecord convertAddDto2Entity(AddMonitorIndicatorRecordDTO addMonitorIndicatorRecordDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param monitorIndicatorRecord 更新前的do对象
    * @param updateMonitorIndicatorRecordDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-10 00:13:27
    */
    MonitorIndicatorRecord convertUpdateDto2Entity(@MappingTarget MonitorIndicatorRecord monitorIndicatorRecord, UpdateMonitorIndicatorRecordDTO updateMonitorIndicatorRecordDTO);
}

