package com.persagy.apm.diagnose.maintenance.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.diagnose.config.RabbitConfig;
import com.persagy.apm.diagnose.constant.AlarmConstants;
import com.persagy.apm.diagnose.indicatorrecord.service.IMonitorIndicatorRecordService;
import com.persagy.apm.diagnose.maintenance.constant.EnumAlarmItem;
import com.persagy.apm.diagnose.maintenance.constant.EnumMaintenanceIndicatorType;
import com.persagy.apm.diagnose.maintenance.model.dto.AlarmProjectDTO;
import com.persagy.apm.diagnose.maintenance.model.dto.CreatePMDataDTO;
import com.persagy.apm.diagnose.service.EnergyAlarmServiceImpl;
import com.persagy.apm.diagnose.service.dto.AlarmRuleDTO;
import com.persagy.apm.diagnose.utils.DateUtils;
import com.persagy.apm.diagnose.utils.RedisLock;
import com.persagy.apm.diagnose.utils.RedisUtil;
import com.persagy.framework.ems.data.pojo.hbase.MonitorIndicatorRecord;
import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.*;

@Slf4j
@Api(tags = "维保数据历史记录")
@Validated
@RestController
@RequestMapping("MaintenanceDataRecords")
public class MaintenanceController {
    @Autowired
    private RedisLock lockUtil;
    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private RabbitConfig rabbitConfig;
    @Autowired
    private EnergyAlarmServiceImpl energyAlarmService;
    @Autowired
    private IMonitorIndicatorRecordService monitorIndicatorRecordService;

    //创建压测 数据
    @PostMapping("/createDataPM")
    public void createPMData(@Valid @RequestBody CreatePMDataDTO createPMDataDTO) throws Exception {
        String startTime = createPMDataDTO.getStartTime();
        String endTime = createPMDataDTO.getEndTime();
        Date startDate = DateUtils.str2Date(startTime, DateUtils.SDF_SECOND);
        Date endDate = DateUtils.str2Date(endTime, DateUtils.SDF_SECOND);
        log.info("开始创造压测测试数据,时间范围:{}~~~{}", startTime, endTime);
        //查询项目列表
        List<AlarmProjectDTO> alarmProjectDTOList = energyAlarmService.getAlarmProjectDTOList();
        int count = 0;
        if (!CollectionUtils.isEmpty(alarmProjectDTOList)) {
            log.info("共有{}个项目需要创建压测测试数据", alarmProjectDTOList.size());
            // 先按项目、再按时间、再按
            for (AlarmProjectDTO projectDTO : alarmProjectDTOList) {
                log.info("添加项目{}的监测值", projectDTO.getProjectId());
                List<AlarmRuleDTO> projectAlarmRules = energyAlarmService.getProjectAlarmRules(projectDTO.getProjectId());
                for (Date start = startDate; start.before(endDate); start = DateUtils.addMinutes(start, 15)) {
                    //通过报警服务查询报警规则
                    List<MonitorIndicatorRecord> recordList = new ArrayList<>();
                    for (AlarmRuleDTO projectAlarmRule : projectAlarmRules) {
                        List<String> objIds = projectAlarmRule.getObjIds();
                        if (CollectionUtils.isEmpty(objIds)) {
                            continue;
                        }
                        List<String> infoCodes = projectAlarmRule.getInfoCodes();
                        if (CollectionUtils.isEmpty(infoCodes)) {
                            continue;
                        }
                        for (String infoCode : infoCodes) {
                            for (String objId : objIds) {

                                int num = (int) (Math.random() * 100);
                                //构造实体对象
                                MonitorIndicatorRecord monitorIndicatorRecord = new MonitorIndicatorRecord();
                                monitorIndicatorRecord.setProject(projectDTO.getProjectId());
                                monitorIndicatorRecord.setObjId(objId);
                                monitorIndicatorRecord.setMonitorIndicatorId(infoCode);
                                monitorIndicatorRecord.setDataTime(start);
                                monitorIndicatorRecord.setDataValue(Double.valueOf(num));
                                recordList.add(monitorIndicatorRecord);
                            }
                        }
                    }
                    long saveIndicatorStartTime = System.currentTimeMillis();
                    monitorIndicatorRecordService.batchSaveMonitorIndicatorRecordList(recordList);
                    log.info("添加{}的监测值", startDate);
                    log.info("共添加{}个监测值，共用时{}ms", recordList.size(), System.currentTimeMillis() - saveIndicatorStartTime);
                }
                count++;
                log.info("已完成{}个项目的压测测试数据创建, 剩余{}个项目的压测测试数据待创建", count, alarmProjectDTOList.size() - count);
                Thread.sleep(5);
            }
        }
    }


    @PostMapping("/sendData")
    public void sendData() throws Exception {

        Map<String, Boolean> iotDataMap = new LinkedHashMap<>();
        iotDataMap.put("10", true);
        iotDataMap.put("11", false);
        iotDataMap.put("12", true);
        iotDataMap.put("13", false);
        iotDataMap.put("14", true);
        iotDataMap.put("15", false);
        iotDataMap.put("16", true);
        iotDataMap.put("17", false);
        iotDataMap.put("18", true);

        String objId = "Eq1101060001f00f2ed121f9407783ac0f0a520a9616", alarmItemCode = EnumAlarmItem.EvapAbnormal.getCode();
        String lockKey = AlarmConstants.alarmDataLockPrefix + objId + "-" + alarmItemCode;
        for (String time : iotDataMap.keySet()) {
            while (true) {
                Object isLock = redisUtil.get(lockKey);
                if (isLock == null) {
                    break;
                }
                Thread.sleep(100);
            }
            long timeLong = System.currentTimeMillis() + (10 * 1000);
            lockUtil.sendDataLock(lockKey, String.valueOf(timeLong));
            JSONArray array = new JSONArray();
            if (iotDataMap.get(time)) {
                array.add(getTriggerData(time, objId, alarmItemCode));
            } else {
                array.add(getNormalData(time, objId, alarmItemCode));
            }
            //推送
            rabbitConfig.sendMsg(array);
        }
    }

    public JSONArray getSendArray(Map<String, Boolean> iotDataMap, String objId, String alarmItemCode) {
        if (iotDataMap == null) {
            return null;
        }
        JSONArray array = new JSONArray();
        for (String time : iotDataMap.keySet()) {
            if (iotDataMap.get(time)) {
                array.add(getTriggerData(time, objId, alarmItemCode));
            } else {
                array.add(getNormalData(time, objId, alarmItemCode));
            }
        }
        return array;
    }

    //触发报警
    public JSONObject getTriggerData(String time, String objId, String alarmItemCode) {
        return initJSONObject("202111" + time + "000000", objId, alarmItemCode, 5.0, 2.0);
    }

    //恢复
    public JSONObject getNormalData(String time, String objId, String alarmItemCode) {
        return initJSONObject("202111" + time + "000000", objId, alarmItemCode, 1.0, 0.0);
    }

    public JSONObject initJSONObject(String dataTime, String objId, String alarmItemCode, Double value1, Double value2) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("dataTime", dataTime);
        jsonObject.put("itemCode", alarmItemCode);
        jsonObject.put("meterId", objId);
        JSONArray funcIdArray = new JSONArray();
        funcIdArray.add(EnumMaintenanceIndicatorType.ACCCCC_EvapHeatTransferDayAvg.getCode());//换热端差日维度平均值阈值
        funcIdArray.add(EnumMaintenanceIndicatorType.ACCCCC_EvapContinuousOperation.getCode());//运行持续时间
        JSONArray valueArray = new JSONArray();
        valueArray.add(value1);
        valueArray.add(value2);
        jsonObject.put("funcId", funcIdArray);
        jsonObject.put("value", valueArray);
        return jsonObject;
    }

}
